@@ -12,7 +12,6 @@ package main
 import (
 	"bufio"
 	"flag"
-	"io/ioutil"
 	"log"
 	"os"
 	"os/exec"
@@ -36,7 +35,7 @@ func main() {
 		if err != nil {
 			log.Fatal(err)
 		}
-		err = exec.Command("go", "run", "golang.org/x/tools/cmd/goimports", "-w", path).Run()
+		err = exec.Command("go", "tool", "goimports", "-w", path).Run()
 		if err != nil {
 			log.Fatal(err)
 		}
@@ -51,7 +50,7 @@ func pruneInterfaceCheck(path string, size int64) error {
 	}
 	defer fd.Close()
 
-	tmp, err := ioutil.TempFile(".", "")
+	tmp, err := os.CreateTemp(".", "")
 	if err != nil {
 		return err
 	}
